"use strict";

/* exported logger */

var AbstractGuiElement = require('../AbstractGuiElement');
var DualListbox = require('dual-listbox').DualListbox;
var GuiConnector = require("../../GuiConnector");
var UserPreferences = require("../../map/data/UserPreferences");

var InvalidArgumentError = require ('../../InvalidArgumentError');

var Functions = require('../../Functions');
var logger = require('../../logger');

var Promise = require("bluebird");

function ChooseAnnotatorsDialog(params) {
  AbstractGuiElement.call(this, params);
  var self = this;
  self.createGui();
}

ChooseAnnotatorsDialog.prototype = Object.create(AbstractGuiElement.prototype);
ChooseAnnotatorsDialog.prototype.constructor = ChooseAnnotatorsDialog;

ChooseAnnotatorsDialog.prototype.createGui = function () {
  var self = this;
  var content = Functions.createElement({
    type: "div",
    style: "display:table"
  });
  content.appendChild(Functions.createElement({
    type: "div",
    style: "display:table-cell",
    content: "<div name='elementTree'/>",
    xss: false
  }));

  var annotatorsDiv = Functions.createElement({
    type: "div",
    style: "display:table-cell",
    xss: false
  });

  annotatorsDiv.appendChild(Functions.createElement({
    type: "div",    
    name: "annotatorListBox"
  }));

  annotatorsDiv.appendChild(Functions.createElement({
    type: "div",    
    className: "minerva-annotators-params"
  }));

  content.appendChild(annotatorsDiv);  

  self.getElement().appendChild(content);
};

function onChangeParameterValue(element, user){  
  var name = $(element).siblings(".minerva-annotator-param-name").text();
  var annotatorClassName = $(element).parent().parent().attr('name');
     
  var data = new UserPreferences();

  var annotatorsParams = {};
  annotatorsParams[annotatorClassName] = {};
  if (element.type == 'checkbox') annotatorsParams[annotatorClassName][name] = element.checked.toString();
  else annotatorsParams[annotatorClassName][name] = element.value;
  data.setAnnotatorsParameters(annotatorsParams);

  return ServerConnector.updateUserPreferences({user: user, preferences: data}).then(null, GuiConnector.alert);  
}

ChooseAnnotatorsDialog.prototype.setElementType = function (elementType) {
  var self = this;

  var configuration;

  return ServerConnector.getConfiguration().then(function (result) {
    configuration = result;
    return ServerConnector.getLoggedUser();
  }).then(function (user) {
    var element = $("[name='annotatorListBox']", self.getElement())[0];
    var annotatorsParams = $(".minerva-annotators-params", self.getElement())[0];
    Functions.removeChildren(element);
    Functions.removeChildren(annotatorsParams);

    var selectElement = Functions.createElement({
      type: "select",
      className: "minerva-multi-select"
    });

    var annotators = configuration.getElementAnnotators(elementType);

    var selectedAnnotators = user.getPreferences().getElementAnnotators(elementType.className);
    var existingAnnotatorsParameters = user.getPreferences().getAnnotatorsParameters();


    for (var i = 0; i < annotators.length; i++) {
      var annotator = annotators[i];
      var selected = false;
      for (var j = 0; j < selectedAnnotators.length; j++) {
        if (annotator.getName() === selectedAnnotators[j]) {
          selected = true;
          
          var paramsDefs = annotator.getParametersDefinitions();
          if (paramsDefs.length > 0) {
            console.log('annotator', annotator);

            annotatorsParams.appendChild(Functions.createElement({
              type: "div",    
              className: "minerva-annotators-params-header",
              content: '<div>Available parameters</div>'
            }))

            var annotatorParams = Functions.createElement({
              type: "div",    
              className: "minerva-annotator-params",
              name: annotator.getClassName()
            });
  
            annotatorParams.appendChild(Functions.createElement({
              type: "div",    
              className: "minerva-annotator-params-header",
              content: annotator.getName()
            }));

            for (var k = 0; k < paramsDefs.length; k++) {
              var param = paramsDefs[k];              
              var paramElement = Functions.createElement({
                type: "div",    
                className: "minerva-annotator-param"
              });

              var paramName = Functions.createElement({
                type: "div",
                className: "minerva-annotator-param-name",
                content: param.name
              });

              console.log('param', param);
              var toolTip = Functions.createElement({
                type: "span",
                className: "annotator-tooltip",
                content: param.description
              });

              paramName.appendChild(toolTip)
              paramElement.appendChild(paramName);    

              var paramValue;

              var existingParamValue;
              if (existingAnnotatorsParameters[annotator.getClassName()]) {
                existingParamValue = existingAnnotatorsParameters[annotator.getClassName()][param.name]
              }

              console.log('existingParamValue', existingParamValue);

              if (param.type.indexOf("String") >= 0) {
                paramValue = Functions.createElement({
                  type: "textarea",
                  onchange: function(){return onChangeParameterValue(this, user);}
                });
                if (existingParamValue) paramValue.value = existingParamValue;
              } else if (param.type.indexOf("Integer") >= 0) {
                paramValue = Functions.createElement({
                  type: "input",
                  inputType: "number",
                  onchange: function(){return onChangeParameterValue(this, user);}
                });
                if (existingParamValue) paramValue.value = existingParamValue;
              } else if (param.type.indexOf("Boolean") >= 0) {
                paramValue = Functions.createElement({
                  type: "input",
                  inputType: "checkbox",
                  onchange: function(){return onChangeParameterValue(this, user);}
                });
                paramValue.checked = (existingParamValue && existingParamValue === 'true');
              } else {
                throw new InvalidAlgorithmError("Unknown annotator parameter type");
              }    
              
              paramElement.appendChild(paramValue);
              annotatorParams.appendChild(paramElement);              
            }

            annotatorsParams.appendChild(annotatorParams);
          }
        }      
      }
      var option = new Option();
      option.value = annotator.getClassName();
      option.attributes.selected = selected;
      option.innerHTML = "<div>" + annotator.getName() + "</div>";
      selectElement.appendChild(option);
    };

    element.appendChild(selectElement);
    new DualListbox(selectElement, {
      addEvent: function (value) {
        var annotators = configuration.getElementAnnotators();
        var annotator;
        for (var i = 0; i < annotators.length; i++) {
          if (value === annotators[i].getClassName()) {
            annotator = annotators[i];
          }
        }
        selectedAnnotators.push(annotator.getName());

        var data = new UserPreferences();

        var elementAnnotators = {};
        elementAnnotators[elementType.className] = selectedAnnotators;
        data.setElementAnnotators(elementAnnotators);
        return ServerConnector.updateUserPreferences({user: user, preferences: data}).then(null, GuiConnector.alert);
      },
      removeEvent: function (value) {
        var annotators = configuration.getElementAnnotators();
        var annotator;
        for (var i = 0; i < annotators.length; i++) {
          if (value === annotators[i].getClassName()) {
            annotator = annotators[i];
          }
        }
        var index = selectedAnnotators.indexOf(annotator.getName());
        if (index > -1) {
          selectedAnnotators.splice(index, 1);
        }

        var data = new UserPreferences();

        var elementAnnotators = {};
        elementAnnotators[elementType.className] = selectedAnnotators;
        data.setElementAnnotators(elementAnnotators);
        return ServerConnector.updateUserPreferences({user: user, preferences: data}).then(null, GuiConnector.alert);
      },
      availableTitle: 'Available',
      selectedTitle: 'Used',
      addButtonText: '>',
      removeButtonText: '<',
      addAllButtonText: '>>',
      removeAllButtonText: '<<'
    });
  });

};

ChooseAnnotatorsDialog.prototype.init = function () {
  var self = this;
  return ServerConnector.getConfiguration().then(function (configuration) {

    var treeData = self.createElementTree(configuration);

    var element = $('[name="elementTree"]', self.getElement());
    element.jstree({
      core: {
        data: treeData
      }
    }).on('ready.jstree', function () {
      element.jstree("open_all");
    }).on("select_node.jstree",
      function (evt, data) {
        return self.setElementType(data.node.data);
      }
    );
  });
};

ChooseAnnotatorsDialog.prototype.createElementTree = function (configuration) {

  var elementTypes = configuration.getElementTypes();
  var reactionTypes = configuration.getReactionTypes();

  var treeNodes = {
    "lcsb.mapviewer.model.map.BioEntity": {
      text: "BioEntity",
      children: []
    }
  };

  var i;
  for (i = 0; i < elementTypes.length; i++) {
    var type = elementTypes[i];
    var name = type.className;
    if (name.indexOf(".") > 0) {
      name = name.substr(name.lastIndexOf(".") + 1);
    }
    treeNodes[type.className] = {
      text: name,
      data: type,
      children: []
    };
  }

  for (i = 0; i < reactionTypes.length; i++) {
    var type = reactionTypes[i];
    treeNodes[type.className] = {
      text: type.name,
      data: type,
      children: []
    };
  }

  for (var treeNodeName in treeNodes) {
    if (treeNodes.hasOwnProperty(treeNodeName)) {
      var treeNode = treeNodes[treeNodeName];
      if (treeNode.data !== undefined) {
        var parentNode = treeNodes[treeNode.data.parentClass];
        parentNode.children.push(treeNode);
      }
    }
  }

  return treeNodes["lcsb.mapviewer.model.map.BioEntity"];
};

ChooseAnnotatorsDialog.prototype.destroy = function () {
  $(this.getElement()).dialog("destroy");
};

ChooseAnnotatorsDialog.prototype.open = function () {
  var self = this;
  var div = self.getElement();
  if (!$(div).hasClass("ui-dialog-content")) {
    $(div).dialog({
      title: "Select annotators",
      modal: true,
      width: window.innerWidth / 2,
      height: window.innerHeight / 2

    });
  }

  $(div).dialog("open");
};

module.exports = ChooseAnnotatorsDialog;
